package dockertag

import (
	"strconv"
	"strings"
)

type Versions []*Version

func MakeVersions(ss ...string) Versions {
	var v Versions
	for _, s := range ss {
		v = append(v, NewVersion(s))
	}
	return v
}

func (v Versions) Len() int {
	return len(v)
}

func (v Versions) Less(i, j int) bool {
	b := v[i].Sup(v[j])
	if b == nil {
		return true
	}
	return !*b
}

func (v Versions) Swap(i, j int) {
	v[i], v[j] = v[j], v[i]
}

func (v Versions) Contains(o *Version) bool {
	for _, v := range v {
		if v.String() == o.String() {
			return true
		}
	}
	return false
}

func (v Versions) Filter(fn func(Version) bool) Versions {
	var out Versions
	if fn == nil {
		return out
	}
	for _, vv := range v {
		if fn(*vv) {
			out = append(out, vv)
		}
	}
	return out
}

func NewVersion(s string) *Version {
	return &Version{s: s}
}

type Version struct {
	sem []int
	s   string
}

func (v *Version) IsSemver() bool {
	if len(v.sem) > 0 {
		return true
	}
	parts := strings.Split(v.s, ".")
	if len(parts) > 3 {
		return false
	}
	sem := make([]int, 3)
	for i, p := range parts {
		vi, err := strconv.Atoi(p)
		if err != nil {
			return false
		}
		sem[i] = vi
	}
	v.sem = sem
	return true
}
func (v *Version) Sup(o *Version) (out *bool) {
	out = new(bool)
	if !v.IsSemver() || !o.IsSemver() {
		return nil
	}
	if v.String() == o.String() {
		return nil
	}
	var v1, v2 *Version
	if len(o.sem) < len(v.sem) {
		v1, v2 = o, v
	} else {
		v1, v2 = v, o
	}
	for i := range v1.sem {
		if v1.sem[i] == v2.sem[i] {
			continue
		}
		if v1.sem[i] > v2.sem[i] {
			*out = v1.s == v.s
			return
		} else {
			*out = v1.s == o.s
			return
		}
	}
	*out = v2.s == v.s
	return
}

func (v Version) Major() int {
	return v.index(0)
}

func (v Version) Minor() int {
	return v.index(1)
}

func (v Version) Patch() int {
	return v.index(2)
}

func (v Version) index(i int) int {
	if len(v.sem) < 3 || i > 2 {
		return 0
	}
	return v.sem[i]
}

func (v Version) String() string {
	if len(v.sem) == 0 && !v.IsSemver() {
		return v.s
	}
	var parts []string
	for _, v := range v.sem {
		parts = append(parts, strconv.Itoa(v))
	}
	return strings.Join(parts, ".")
}

type Tags []Tag

func MakeTags(s ...string) Tags {
	var ts Tags
	for _, v := range s {
		ts = append(ts, Tag(v))
	}
	return ts
}

func (t Tags) Filter(fn func(Tag) bool) Tags {
	var out Tags
	if fn == nil {
		return out
	}
	for _, v := range t {
		if fn(v) {
			out = append(out, v)
		}
	}
	return out
}

type Tag string

func (t Tag) IsBeta() bool {
	return strings.Contains(t.toLower(), "beta")
}

func (t Tag) IsRC() bool {
	return strings.Contains(t.toLower(), "rc")
}

func (t Tag) HasSuffix() bool {
	return t.Suffix() != ""
}

func (t Tag) Suffix() string {
	parts := strings.Split(t.String(), "-")
	if len(parts) < 1 {
		return ""
	}
	return strings.Join(parts, "-")
}

func (t Tag) Version() *Version {
	return &Version{s: strings.Split(t.String(), "-")[0]}
}

func (t Tag) String() string {
	return string(t)
}

func (t Tag) Contains(s string) bool {
	return strings.Contains(t.String(), s)
}

func (t Tag) toLower() string {
	return strings.ToLower(t.String())
}
